"""Fixtures provided by homeassistant.core"""
from homeassistant.exceptions import ServiceNotFound
import pytest

from .aiohttp_mock import mock_aiohttp_client

from .common import (  # noqa: E402, isort:skip
    async_test_home_assistant,
    mock_storage as mock_storage,
)


@pytest.fixture
def hass_storage():
    """Fixture to mock storage."""
    with mock_storage() as stored_data:
        yield stored_data


@pytest.fixture
def hass(loop, hass_storage, request):
    """Fixture to provide a test instance of Home Assistant."""

    def exc_handle(loop, context):
        """Handle exceptions by rethrowing them, which will fail the test."""
        exceptions.append(context["exception"])
        orig_exception_handler(loop, context)

    exceptions = []
    hass = loop.run_until_complete(async_test_home_assistant(loop))
    orig_exception_handler = loop.get_exception_handler()
    loop.set_exception_handler(exc_handle)

    yield hass

    loop.run_until_complete(hass.async_stop(force=True))
    for ex in exceptions:
        if isinstance(ex, ServiceNotFound):
            continue
        raise ex


@pytest.fixture
def aioclient_mock():
    """Fixture to mock aioclient calls."""
    with mock_aiohttp_client() as mock_session:
        yield mock_session



"""Mock utilities that are async aware."""
import sys

if sys.version_info[:2] < (3, 8):
    from asynctest.mock import *  # noqa
    from asynctest.mock import CoroutineMock as AsyncMock  # noqa
else:
    from unittest.mock import *  # noqa



"""pytest setup."""
from aioresponses import aioresponses
from homeassistant.exceptions import ServiceNotFound
import pytest

from .aiohttp_mock import mock_aiohttp_client
from .async_mock import Mock

from .common import (  # noqa: E402, isort:skip
    async_test_home_assistant,
    mock_storage as mock_storage,
)


@pytest.fixture
def hass_storage():
    """Fixture to mock storage."""
    with mock_storage() as stored_data:
        yield stored_data


@pytest.fixture
def hass(loop, hass_storage, request):
    """Fixture to provide a test instance of Home Assistant."""

    def exc_handle(loop, context):
        """Handle exceptions by rethrowing them, which will fail the test."""
        exceptions.append(context["exception"])
        orig_exception_handler(loop, context)

    exceptions = []
    hass = loop.run_until_complete(async_test_home_assistant(loop))
    orig_exception_handler = loop.get_exception_handler()
    loop.set_exception_handler(exc_handle)

    yield hass

    loop.run_until_complete(hass.async_stop(force=True))
    for ex in exceptions:
        if isinstance(ex, ServiceNotFound):
            continue
        raise ex


@pytest.fixture
def aioclient_mock():
    """Fixture to mock aioclient calls."""
    with mock_aiohttp_client() as mock_session:
        yield mock_session


@pytest.fixture
def mock_aioresponse():
    """Fixture to mock aiohttp calls."""
    with aioresponses() as m:
        yield m


@pytest.fixture
def coordinator_mock(hass):
    """Fixture to mock the update data coordinator."""
    coordinator = Mock(data={}, hass=hass)
    coordinator.data = {
        "870780": {
            "name": "Control",
            "capsule": "https://steamcdn-a.akamaihd.net/steam/apps/870780/header_292x136.jpg?t=1572428374",
            "review_score": 0,
            "review_desc": "No user reviews",
            "reviews_total": "0",
            "reviews_percent": 0,
            "release_date": "1556236800",
            "release_string": "Coming August 2020",
            "platform_icons": '<span class="platform_img win"></span>',
            "subs": [],
            "type": "Game",
            "screenshots": [
                "ss_4cd872ed17a77fee5ddfff766f56a152968030cf.jpg",
                "ss_5a4d1a304f214a47ca110ceb1172cccaaf997608.jpg",
                "ss_6b486bb45eea618c0a7fd0781202f3ae3315aa91.jpg",
                "ss_91b9bb3feeac710f9497de84392b1f7487989669.jpg",
            ],
            "review_css": "no_reviews",
            "priority": 0,
            "added": 1584991718,
            "background": "https://steamcdn-a.akamaihd.net/steam/apps/274520/page_bg_generated_v6b.jpg?t=1568727996",
            "rank": 1000,
            "tags": [],
            "is_free_game": False,
            "win": 1,
        },
        "952060": {
            "name": "RESIDENT EVIL 3",
            "capsule": "https://steamcdn-a.akamaihd.net/steam/apps/952060/header_292x136.jpg?t=1590098547",
            "review_score": 6,
            "review_desc": "Mostly Positive",
            "reviews_total": "17,331",
            "reviews_percent": 73,
            "release_date": "1585886220",
            "release_string": "Apr 2",
            "platform_icons": '<span class="platform_img win"></span>',
            "subs": [
                {
                    "id": 437203,
                    "discount_block": '<div class="discount_block discount_block_large no_discount" data-price-final="5999"><div class="discount_prices"><div class="discount_final_price">$59.99</div></div></div>',
                    "discount_pct": 0,
                    "price": 5999,
                }
            ],
            "type": "Game",
            "screenshots": [
                "ss_77eda710487b89293f109cf7dcf96b4ffab0d1a1.jpg",
                "ss_34f01910d65fb171a27e058cb74623c0eb53ba69.jpg",
                "ss_bec8b7cef716135ea5bbd726a3342ed9ca475b31.jpg",
                "ss_4f6eaac14b8e02c0a68a9c9f7627dd84cde1abb8.jpg",
            ],
            "review_css": "positive",
            "priority": 0,
            "added": 1585503915,
            "background": "https://steamcdn-a.akamaihd.net/steam/apps/952060/page_bg_generated_v6b.jpg?t=1590098547",
            "rank": 713,
            "tags": [],
            "is_free_game": False,
            "win": 1,
        },
        "975150": {
            "name": "Resolutiion",
            "capsule": "https://steamcdn-a.akamaihd.net/steam/apps/975150/header_292x136.jpg?t=1590678003",
            "review_score": 0,
            "review_desc": "3 user reviews",
            "reviews_total": "3",
            "reviews_percent": 100,
            "release_date": "1590677842",
            "release_string": "May 28",
            "platform_icons": '<span class="platform_img win"></span><span class="platform_img mac"></span><span class="platform_img linux"></span>',
            "subs": [
                {
                    "id": 320073,
                    "discount_block": '<div class="discount_block discount_block_large" data-price-final="1699"><div class="discount_pct">-15%</div><div class="discount_prices"><div class="discount_original_price">$19.99</div><div class="discount_final_price">$16.99</div></div></div>',
                    "discount_pct": 15,
                    "price": 1699,
                }
            ],
            "type": "Game",
            "screenshots": [
                "ss_08e312cea81092226c8e20c57b4d578ecd4ffa3a.jpg",
                "ss_66d6693d1e84b2a0932ec3b88f54901eebdac68f.jpg",
                "ss_e11b14c5e5e6e55bf8fbd064e4a9a8e1b49e528f.jpg",
                "ss_3405c0136e4c11538c54a0675bea60196aaf66e3.jpg",
            ],
            "review_css": "not_enough_reviews",
            "priority": 0,
            "added": 1590721175,
            "background": "https://steamcdn-a.akamaihd.net/steam/apps/1057090/page_bg_generated_v6b.jpg?t=1588789337",
            "rank": 830,
            "tags": [],
            "is_free_game": False,
            "win": 1,
            "mac": 1,
            "linux": 1,
        },
    }
    yield coordinator


@pytest.fixture
def manager_mock(coordinator_mock):
    """Pytest fixture mocking the sensor manager class."""
    manager = Mock(coordinator=coordinator_mock)
    yield manager



import asyncio
from contextlib import contextmanager
import functools as ft
import json
import logging
import os

from homeassistant import auth, config_entries, core as ha
from homeassistant.auth import auth_store
from homeassistant.const import EVENT_HOMEASSISTANT_CLOSE
from homeassistant.helpers import storage
import homeassistant.util.dt as date_util
from homeassistant.util.unit_system import METRIC_SYSTEM

from .async_mock import AsyncMock, Mock, patch

_LOGGER = logging.getLogger(__name__)
INSTANCES = []


def get_test_config_dir(*add_path):
    """Return a path to a test config dir."""
    return os.path.join(os.path.dirname(__file__), "testing_config", *add_path)


# pylint: disable=protected-access
async def async_test_home_assistant(loop):
    """Return a Home Assistant object pointing at test config dir."""
    hass = ha.HomeAssistant(loop)
    store = auth_store.AuthStore(hass)
    hass.auth = auth.AuthManager(hass, store, {}, {})
    ensure_auth_manager_loaded(hass.auth)
    INSTANCES.append(hass)

    orig_async_add_job = hass.async_add_job
    orig_async_add_executor_job = hass.async_add_executor_job
    orig_async_create_task = hass.async_create_task

    def async_add_job(target, *args):
        """Add job."""
        check_target = target
        while isinstance(check_target, ft.partial):
            check_target = check_target.func

        if isinstance(check_target, Mock) and not isinstance(target, AsyncMock):
            fut = asyncio.Future()
            fut.set_result(target(*args))
            return fut

        return orig_async_add_job(target, *args)

    def async_add_executor_job(target, *args):
        """Add executor job."""
        check_target = target
        while isinstance(check_target, ft.partial):
            check_target = check_target.func

        if isinstance(check_target, Mock):
            fut = asyncio.Future()
            fut.set_result(target(*args))
            return fut

        return orig_async_add_executor_job(target, *args)

    def async_create_task(coroutine):
        """Create task."""
        if isinstance(coroutine, Mock) and not isinstance(coroutine, AsyncMock):
            fut = asyncio.Future()
            fut.set_result(None)
            return fut

        return orig_async_create_task(coroutine)

    hass.async_add_job = async_add_job
    hass.async_add_executor_job = async_add_executor_job
    hass.async_create_task = async_create_task

    hass.config.location_name = "test home"
    hass.config.config_dir = get_test_config_dir()
    hass.config.latitude = 32.87336
    hass.config.longitude = -117.22743
    hass.config.elevation = 0
    hass.config.time_zone = date_util.get_time_zone("US/Pacific")
    hass.config.units = METRIC_SYSTEM
    hass.config.skip_pip = True

    hass.config_entries = config_entries.ConfigEntries(hass, {})
    hass.config_entries._entries = []
    hass.config_entries._store._async_ensure_stop_listener = lambda: None

    hass.state = ha.CoreState.running

    # Mock async_start
    orig_start = hass.async_start

    async def mock_async_start():
        """Start the mocking."""
        # We only mock time during tests and we want to track tasks
        with patch("homeassistant.core._async_create_timer"), patch.object(
            hass, "async_stop_track_tasks"
        ):
            await orig_start()

    hass.async_start = mock_async_start

    @ha.callback
    def clear_instance(event):
        """Clear global instance."""
        INSTANCES.remove(hass)

    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_CLOSE, clear_instance)

    return hass


@ha.callback
def ensure_auth_manager_loaded(auth_mgr):
    """Ensure an auth manager is considered loaded."""
    store = auth_mgr._store
    if store._users is None:
        store._set_defaults()


@contextmanager
def mock_storage(data=None):
    """Mock storage.
    Data is a dict {'key': {'version': version, 'data': data}}
    Written data will be converted to JSON to ensure JSON parsing works.
    """
    if data is None:
        data = {}

    orig_load = storage.Store._async_load

    async def mock_async_load(store):
        """Mock version of load."""
        if store._data is None:
            # No data to load
            if store.key not in data:
                return None

            mock_data = data.get(store.key)

            if "data" not in mock_data or "version" not in mock_data:
                _LOGGER.error('Mock data needs "version" and "data"')
                raise ValueError('Mock data needs "version" and "data"')

            store._data = mock_data

        # Route through original load so that we trigger migration
        loaded = await orig_load(store)
        _LOGGER.info("Loading data for %s: %s", store.key, loaded)
        return loaded

    def mock_write_data(store, path, data_to_write):
        """Mock version of write data."""
        _LOGGER.info("Writing data to %s: %s", store.key, data_to_write)
        # To ensure that the data can be serialized
        data[store.key] = json.loads(json.dumps(data_to_write, cls=store._encoder))

    async def mock_remove(store):
        """Remove data."""
        data.pop(store.key, None)

    with patch(
        "homeassistant.helpers.storage.Store._async_load",
        side_effect=mock_async_load,
        autospec=True,
    ), patch(
        "homeassistant.helpers.storage.Store._write_data",
        side_effect=mock_write_data,
        autospec=True,
    ), patch(
        "homeassistant.helpers.storage.Store.async_remove",
        side_effect=mock_remove,
        autospec=True,
    ):
        yield data



"""A pytest plugin for use with homeassistant custom components.

This plugin provides pytest fixtures and common testing utils to make testing custom
components easier.  The fixtures and test utils come from the homeassistant core project.
"""
from .fixtures import aioclient_mock, hass, hass_storage  # noqa: F401



#!/usr/bin/env python

import codecs
import os

from setuptools import setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding="utf-8").read()


REQUIREMENTS = [
    "homeassistant",
    "asynctest==0.13.0",
    "astroid==2.3.3",
    "mock-open==1.4.0",
    "pytest-aiohttp==0.3.0",
    "pytest-timeout==1.3.4",
    "pytest==5.4.2",
    "requests_mock==1.8.0",
    "responses==0.10.6",
]

setup(
    name="pytest-homeassistant",
    version="0.1.0",
    author="Aaron Godfrey",
    maintainer="Aaron Godfrey",
    license="MIT",
    url="https://github.com/boralyl/pytest-homeassistant",
    description="A pytest plugin for use with homeassistant custom components.",
    long_description=read("README.rst"),
    packages=["pytest_homeassistant"],
    python_requires=">=3.6, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    install_requires=REQUIREMENTS,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={"pytest11": ["homeassistant = pytest_homeassistant.plugin"]},
)
